/*
 * Project: Bluetooth Net Monitor
 * Author: Zak Kemble, contact@zakkemble.co.uk
 * Copyright: (C) 2013 by Zak Kemble
 * License: GNU GPL v3 (see License.txt)
 * Web: http://blog.zakkemble.co.uk/bluetooth-net-monitor-v2/
 */

#define _BSD_SOURCE
#define _POSIX_C_SOURCE 200809L

#include <stdlib.h>
#include <sys/types.h>
#include <ifaddrs.h>
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <net/if.h>
#include "typedefs.h"
#include "iface.h"
#include "config.h"

static uint32_t ip;
static char gateway[NI_MAXHOST];
static char* interface;

// http://man7.org/linux/man-pages/man3/getifaddrs.3.html

void iface_init()
{
	config_val("interface", &interface);
}

void iface_end()
{
	free(interface);
}

void iface_update()
{
	struct ifaddrs* ifaddr;
	struct ifaddrs* ifa;
	char host[NI_MAXHOST];
	bool found = false;

	getifaddrs(&ifaddr);

	// FIX: duplicate interfaces appear without IP or gateway set

	// Loop through interfaces
	for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next)
	{
		// Is this the interface we're looking for?
		if(strcmp(ifa->ifa_name, interface) != 0)
			continue;

		// Yes!
		found = true;

		// Get IP
		if(ifa->ifa_addr != NULL)
			getnameinfo(ifa->ifa_addr, sizeof(struct sockaddr_in), host, NI_MAXHOST, NULL, 0, NI_NUMERICHOST);
		else
			strcpy(host, "0.0.0.0");

		// Get gateway
		if(ifa->ifa_ifu.ifu_dstaddr != NULL)
			getnameinfo(ifa->ifa_ifu.ifu_dstaddr, sizeof(struct sockaddr_in), gateway, NI_MAXHOST, NULL, 0, NI_NUMERICHOST);
		else
			strcpy(gateway, "0.0.0.0");

//		printf("%s %s %s\n", ifa->ifa_name, host, gateway);
	}

	freeifaddrs(ifaddr);

	// Interface not found? Then set everything to 0
	if(!found)
	{
		strcpy(host, "0.0.0.0");
		strcpy(gateway, "0.0.0.0");
	}

	// Convert IP string to int32
	struct in_addr ip_addr;
	inet_pton(AF_INET, host, &ip_addr);
	ip = ip_addr.s_addr;
}

uint32_t iface_ip()
{
	return ip;
}

char* iface_gateway()
{
	return gateway;
}

/*
// Source: http://www.geekpage.jp/en/programming/linux-network/get-ipaddr.php
static uint32_t getIP()
{
	struct ifreq ifr;

	// Open socket
	int fd = socket(AF_INET, SOCK_DGRAM, 0);

	// IPv4 address (TODO: IPv6?)
	ifr.ifr_addr.sa_family = AF_INET;

	// Which interface
	strncpy(ifr.ifr_name, interface, IFNAMSIZ-1);

	// 
	ioctl(fd, SIOCGIFADDR, &ifr);

	// Close socket
	close(fd);

	return ((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr;
}
*/