/*
 * Project: Bluetooth Net Monitor
 * Author: Zak Kemble, contact@zakkemble.co.uk
 * Copyright: (C) 2013 by Zak Kemble
 * License: GNU GPL v3 (see License.txt)
 * Web: http://blog.zakkemble.co.uk/bluetooth-net-monitor-v2/
 */

#define _BSD_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include "typedefs.h"
#include "serial.h"
#include "config.h"

static void openSerial(void);
static void set_interface_attribs(int, int);

static int fd;
static char* port;
static speed_t baud;

void serial_init()
{
	char* tmpBaud = NULL;

	config_val("serial", &port);
	config_val("baud", &tmpBaud);

	int baud = atoi(tmpBaud);

	free(tmpBaud);

	// Change this to a table lookup?
	switch(baud)
	{
		case 0:
			baud = B4800;
			break;
		case 1:
			baud = B9600;
			break;
		case 2:
			baud = B19200;
			break;
		case 3:
			baud = B38400;
			break;
		case 4:
			baud = B57600;
			break;
		case 5:
			baud = B115200;
			break;
		default:
			baud = B9600;
			break;
	}

	openSerial();
}

void serial_end()
{
	free(port);
	close(fd);
}

void serial_send(byte* data, size_t len)
{
	ssize_t res = write(fd, data, len);

	// Error occurred, try closing and opening port
	if(res < 0)
	{
		close(fd);
		openSerial();
		// Try writing again?
	}
}

static void openSerial()
{
	// Open port
	fd = open(port, O_RDWR | O_NOCTTY | O_SYNC);
	if(fd < 0)
		return;

	set_interface_attribs(baud, 0);
/*
	// Non-blocking
	fcntl(fd, F_SETFL, O_NONBLOCK);

	// Attributes
	struct termios tty;
	memset(&tty, 0, sizeof(tty));

	// Get current port attributes
	if(tcgetattr(fd, &tty) != 0)
		return;

	// Set baud
	cfsetospeed(&tty, baud);
	cfsetispeed(&tty, baud);

	cfmakeraw(&tty);

	// Set new attributes
	if(tcsetattr(fd, TCSANOW, &tty) != 0)
		return;
*/
}

// http://stackoverflow.com/questions/6947413/how-to-open-read-and-write-from-serial-port-in-c
static void set_interface_attribs(int speed, int parity)
{
	// Attributes
	struct termios tty;
	memset(&tty, 0, sizeof(tty));

	// Get current port attributes
	if(tcgetattr(fd, &tty) != 0)
		return;

	// Set baud
	cfsetospeed(&tty, speed);
	cfsetispeed(&tty, speed);

	tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS8;     // 8-bit chars
	// disable IGNBRK for mismatched speed tests; otherwise receive break
	// as \000 chars
	tty.c_iflag &= ~IGNBRK;         // ignore break signal
	tty.c_lflag = 0;                // no signaling chars, no echo,
	// no canonical processing
	tty.c_oflag = 0;                // no remapping, no delays
	tty.c_cc[VMIN]  = 0;            // read doesn't block
	tty.c_cc[VTIME] = 5;            // 0.5 seconds read timeout

	tty.c_iflag &= ~(IXON | IXOFF | IXANY); // shut off xon/xoff ctrl

	tty.c_cflag |= (CLOCAL | CREAD); // ignore modem controls,
	// enable reading
	tty.c_cflag &= ~(PARENB | PARODD); // shut off parity
	tty.c_cflag |= parity;
	tty.c_cflag &= ~CSTOPB;
	tty.c_cflag &= ~CRTSCTS;

	// Set new attributes
	tcsetattr(fd, TCSANOW, &tty);
}
